<?php

namespace Drupal\layout_paragraphs_clipboard\Controller;

use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\layout_paragraphs\LayoutParagraphsLayout;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutRefreshTrait;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository;
use Drupal\paragraphs_clipboard\ParagraphsClipboardService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class CopyClipboardController.
 *
 * Copy and Paste a component of a Layout Paragraphs Layout.
 */
class CopyClipboardController extends ControllerBase {

  use LayoutParagraphsLayoutRefreshTrait;
  use AjaxHelperTrait;

  /**
   * The tempstore service.
   *
   * @var \Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository
   */
  protected $tempstore;

  /**
   * The clipboard service.
   *
   * @var \Drupal\paragraphs_clipboard\ParagraphsClipboardService
   */
  protected $clipboardService;

  /**
   * {@inheritDoc}
   */
  public function __construct(LayoutParagraphsLayoutTempstoreRepository $tempstore, ParagraphsClipboardService $clipboard_service) {
    $this->tempstore = $tempstore;
    $this->clipboardService = $clipboard_service;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_paragraphs.tempstore_repository'),
      $container->get('paragraphs_clipboard.paragraphs_clipboard_service')
    );
  }

  /**
   * Duplicates a component and returns appropriate response.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout_paragraphs_layout
   *   The layout paragraphs layout object.
   * @param string $source_uuid
   *   The source component to be cloned.
   *
   * @return array|\Drupal\Core\Ajax\AjaxResponse
   *   A build array or Ajax respone.
   */
  public function copyClipboard(LayoutParagraphsLayout $layout_paragraphs_layout, string $source_uuid) {
    $this->setLayoutParagraphsLayout($layout_paragraphs_layout);
    $clipboard_data = [
      'paragraph_uuid' => $source_uuid,
    ];
    $this->clipboardService->setClipboardData(ParagraphsClipboardService::CLIPBOARD_UUID_KEY, $clipboard_data);

    if ($this->isAjax()) {
      $response = new AjaxResponse();
      if ($this->needsRefresh()) {
        return $this->refreshLayout($response);
      }

      $response->addCommand(new MessageCommand('Paragraph has been copied to clipboard.', NULL, [], FALSE));
      $response->setAttachments([
        'library' => ['layout_paragraphs_clipboard/builder'],
      ]);
      return $response;
    }
    return [
      '#type' => 'layout_paragraphs_builder',
      '#layout_paragraphs_layout' => $layout_paragraphs_layout,
    ];
  }

}
